@@ -99,7 +99,7 @@ class ScheduledEventLocation:
             self.value = value
 
     def __repr__(self) -> str:
-        return f"<ScheduledEventLocation value={self.value} type={self.type}>"
+        return f"<ScheduledEventLocation value={self.value!r} type={self.type}>"
 
     def __str__(self) -> str:
         return str(self.value)
@@ -255,7 +255,7 @@ class ScheduledEvent(Hashable):
         return Asset._from_scheduled_event_cover(
             self._state,
             self.id,
-            self._image,
+            self._cover,
         )
 
     async def edit(
